@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2014 Jens Kasten. All Rights Reserved.
+# Copyright (C) 2011-2016 Jens Kasten. All Rights Reserved.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@ Copyright (C) Jens Kasten. All Rights Reserved.
 
 __author__  = "Jens Kasten <jens@kasten-edv.de>"
 __status__  = "beta"
-__version__ = "0.4"
+__version__ = "0.5"
 __date__    = "08 July 2014"
 
 __all__ = ["RunJail", ]
@@ -54,10 +54,7 @@ RSBAC_JAIL_ATTRIBUTE = [
     'jail_scd_modify'
 ]
 
-
 # coresponde with the jail config keys
-# { "jail-flags": ["allow-dev-read"] }
-# call in rsbactools/jail/helper.py the class JailFlags
 RSBAC_JAIL_CLASS_MAPPER = {
     "jail-flags": "JailFlags",
     "max-caps": "JailMaxCaps",
@@ -69,8 +66,6 @@ RSBAC_JAIL_CLASS_MAPPER = {
     "chroot": "JailChroot",
 }
 
-
-
 # jail flags value are taken from 
 # rsbac/types.h line around 462
 RSBAC_JAIL_FLAGS = {
@@ -90,11 +85,14 @@ RSBAC_JAIL_FLAGS = {
     "this-is-syslog": (32768, "-Y"),
     "allow-ipc-syslog": (65536, "-y"),
     "allow-netlink": (131072, "-K"),
-    "allow-netlink-mod-system": (262142, "-k"),
+    "allow-netdev-mod-system": (262144, "-F"),
+    "allow-process-by-parent": (524288, "-T"),
     "virtual-user": ("virtual-user", "-V"),
     "private-namespace": ("private-namsespace", "-N"),
 }
        
+# take from linux kernel source
+# include/uapi/capabilities.h
 RSBAC_JAIL_MAX_CAPS = {
     "chown": (0, "CHOWN"),
     "dac-override": (1, "DAC_OVERRIDE"),
@@ -128,6 +126,12 @@ RSBAC_JAIL_MAX_CAPS = {
     "audit-control": (30, "AUDIT_CONTROL"),
     "audit-write": (29, "AUDIT_WRITE"),
     "setfcap": (31, "SETFCAP"),
+    "mac-override": (32, "MAC_OVERRIDE"),
+    "mac-admin": (33, "MAC_ADMIN"),
+    "syslog": (34, "SYSLOG"),
+    "wake-alarm": (35, "WAKE_ALARM"),
+    "block-suspend": (36, "BLOCK_SUSPEND"),
+    "audit-read": (37, "AUDIT_READ")
 }
 
 # cap flags are taken from 
@@ -159,6 +163,9 @@ RSBAC_JAIL_SCD = {
     "capability": (22, "capability"),
     "kexec": (23, "kexec"),
     "videomem": (24, "videomem"),
+    "bpf": (25, "bpf"),
+    "perf": (26, "perf"),
+    "none": (27, "none")
 }
 
 
@@ -212,10 +219,10 @@ class JailMaxCaps(object):
                 self.params.append(RSBAC_JAIL_MAX_CAPS[i][1])
             except KeyError as error:
                 log.error(error)
-                sys.stderr.write("Not valid value in '%s' for 'max-caps'" % config_file)
-                sys.stderr.write("Available max-caps:")
+                sys.stderr.write("Not valid value in '%s' for 'max-caps\n'" % config_file)
+                sys.stderr.write("Available max-caps:\n")
                 for i in sorted(RSBAC_JAIL_MAX_CAPS):
-                    sys.stderr.write("%-21s %-21s" % ("", i))
+                    sys.stderr.write("%-21s %-21s \n" % ("", i))
                 sys.exit(22)
         return self.params
 
@@ -231,10 +238,10 @@ class JailScdGet(object):
                 self.params.append(RSBAC_JAIL_SCD[i][1])
             except KeyError as error:
                 log.error(error)
-                sys.stderr.write("Not valid value in '%s' for 'scd-get'" % config_file)
-                sys.stderr.write("Available scd-get:")
+                sys.stderr.write("Not valid value in '%s' for 'scd-get'\n" % config_file)
+                sys.stderr.write("Available scd-get:\n")
                 for i in sorted(RSBAC_JAIL_SCD):
-                    sys.stderr.write("%-21s %-21s" % ("", i))
+                    sys.stderr.write("%-21s %-21s\n" % ("", i))
                 sys.exit(23)
         return self.params
 
@@ -250,10 +257,10 @@ class JailScdModify(object):
                 self.params.append(RSBAC_JAIL_SCD[i][1])
             except KeyError as error:
                 log.error(error)
-                sys.stderr.write("Not valid value in '%s' for 'scd-modify'" % config_file)
-                sys.stderr.write("Available scd-modify:")
+                sys.stderr.write("Not valid value in '%s' for 'scd-modify'\n" % config_file)
+                sys.stderr.write("Available scd-modify:\n")
                 for i in sorted(RSBAC_JAIL_SCD):
-                    sys.stderr.write("%-21s %-21s" % ("", i))
+                    sys.stderr.write("%-21s %-21s\n" % ("", i))
                 sys.exit(24)
         return self.params
 
@@ -292,11 +299,12 @@ class RunJail(object):
         self.args = args
         self.policies = {}
         self.params = []
+       
         if "dry_run" in args and args["dry_run"]:
             self.set_log_level(logging.INFO)
         if "verbose" in args and args["verbose"]:
             self.set_log_level(logging.DEBUG)
-
+    
     def set_log_level(self, log_level):
         log.setLevel(log_level)
 
@@ -311,8 +319,6 @@ class RunJail(object):
             self.policies = jl.get_values()
             log.debug("loading values: %s" % self.policies)
             return True
-        else:
-            return False
 
     def get_policies(self):
         return self.policies
@@ -320,9 +326,7 @@ class RunJail(object):
     def set_params(self, config_file):
         p = JailParams()
         p.set_log_level(self.get_log_level())
-        if not p.build(self.get_policies(), config_file):
-            return False
-        else:
+        if p.build(self.get_policies(), config_file):
             self.params = p.get()
             log.debug("converted values: %s" % self.params)
             return True
@@ -330,51 +334,66 @@ class RunJail(object):
     def get_params(self):
         return self.params 
 
+    def prepare(self):
+       if self.args.get("config_file", False):
+            config_file = os.path.join(RSBAC_JAIL_CONFIG_DIR, 
+                                       ".".join([self.args["config_file"], 
+                                                "json"]))
+            if not self.set_policies(config_file):
+                sys.exit(1)
+            if not self.set_params(config_file):
+                return False
+            return True
+
+
     def run(self):
-        args = []
-        if not "program" in self.args:
-            log.error("missing key 'program' in args")
-            return False
+        self.prepare()
+     
+        rsbac_jail = which("rsbac_jail")
+        program = which(self.args.get("program", False)[0])
+        program_params = self.args.get("program", "")[1:] 
+
+        if not rsbac_jail: 
+            print("command not found: rsbac_jail")
+            sys.exit(1)
+        if not program:
+            print("command not found: %s" % self.args.get("program")[0])
+            sys.exit(1)
+
         if rsbac.Rsbac().get_module("JAIL") == "on":
             log.debug("running in jail mode")
-            prog = which("rsbac_jail")
-            if not prog:
-                sys.stderr.write("Could not found: %s" % rsbac_jail)
-                sys.exit(10)
-            args.append(prog)
-            if "config_file" in self.args and self.args["config_file"]:
-                config_file = os.path.join(RSBAC_JAIL_CONFIG_DIR, 
-                    ".".join([self.args["config_file"], "json"]))
-                if not self.set_policies(config_file):
-                    return False
-                if not self.set_params(config_file):
-                    return False
-                args += self.get_params()
+            jail = True
         else:
-            log.debug("running not in jail mode")
-            prog = which(self.args["program"][0])
-            if not prog:
-                sys.stderr.write("Could not found: %s" % self.args["program"][0])
-                sys.exit(11)
-        log.debug("Program to execute: %s" % prog)
-        prog_to_jail = which(self.args["program"][0])
-        if not prog_to_jail:
-            sys.stderr.write("Could not found: %s" % self.args["program"][0])
-            sys.exit(11)
-        args.append(prog_to_jail)
-        args += self.args["program"][1:]
-        log.debug("Command to execute: %s" % args)
+            log.debug("not running in jail mode")
+            jail = False
        
-        if "dry_run" in self.args and self.args["dry_run"]:
-            log.info(" ".join([i for i in args]))
-            return True
-        try:
-            os.execv(prog, args)
-            return True
-        except OSError as error:
-            log.error(error)
-            sys.stderr.write(error)
-            sys.exit(1)
+        args = []
+        if jail or self.args.get("ignore"):
+            prog = rsbac_jail
+            args.append(prog)
+            args.extend(self.get_params())
+            args.append(program)
+            args.extend(program_params)
+        else:
+            prog = program
+            args.append(prog)
+            args.extend(program_params)
+
+        if self.args.get("show"):
+            output = rsbac_jail + " " + " ".join([i for i in self.get_params()])
+            output += " " + program +  " " + " ".join([str(i) for i in program_params])
+            print(output)
+            sys.exit(0)
+
+        elif self.args.get("dry_run"):
+            log.info("args: %s" % self.args)
+            log.info("this would execute through os.excev()")
+            log.info("%s %s" % (prog, args))
+        else:
+            try:
+                os.execv(prog, args)
+            except OSError as error:
+                log.error(error)
 
 
 class PsJail(object):
@@ -389,7 +408,7 @@ class PsJail(object):
         """Search for pid and add them to dictionary."""
         pid = get_pid_from_name(prog_name)
         if pid:
-            self.processes[str(pid)] = prog_name
+            self.processes[pid] = prog_name
             return True
         else:
             return False
@@ -415,6 +434,7 @@ class PsJail(object):
 
         # collect all jail infos 
         for pid, process_name in self.processes.items():
+            counter = int(pid)
             # first get jail_id to test if further action is needed
             jail_id, error = rsbac_api.attr_get_process("JAIL", pid, "jail_id")
             if error:
@@ -426,7 +446,8 @@ class PsJail(object):
             if jail_id_only and jail_id_only != int(jail_id):
                 continue
             # assign values to dict    
-            self.processes_info[process_name] = {
+            self.processes_info[counter] = {
+                "process_name": process_name,
                 "jail_id": jail_id,
                 "pid": pid,
             }
@@ -439,11 +460,11 @@ class PsJail(object):
                 if error:
                     continue
                 if attribute == "jail_ip":
-                    self.processes_info[process_name][attribute] = attr.strip()
+                    self.processes_info[counter][attribute] = attr.strip()
                 elif int(attr) > 0:
                     # get human readable names
                     names = self.get_names_from_number(attribute, attr)
-                    self.processes_info[process_name][attribute] = names
+                    self.processes_info[counter][attribute] = names
 
     def get_names_from_number(self, attribute, number):
         """Return a list of human readable names."""
@@ -470,6 +491,7 @@ class PsJail(object):
     def show(self):
         """Show the collection information on a terminal with 80 char wide."""
         header = {}
+        header["chroot"] = "Chroot"
         header["jail_flags"] = "Jail-Flags"
         header["jail_max_caps"] = "Jail-Max-Caps"
         header["jail_scd_get"] = "Jail-SCD-Get"
@@ -477,13 +499,17 @@ class PsJail(object):
 
         if len(self.processes_info.items()) == 0:
             return 
-        
-        for process_name, value in sorted(self.processes_info.items()):
+       
+        for key in sorted(self.processes_info.items()):
+            value = key[1]
             print("-" * 79)
             print("%-18s  Pid: %-6s  Jail Id: %-6s  Jail Ip: %-16s" % (
-                process_name, value["pid"], value["jail_id"], value["jail_ip"]))
+                value["process_name"], value["pid"], value["jail_id"], value["jail_ip"]))
             print("-" * 79)
 
+            if "chroot" in value:
+                print("%-15s: %s" % (header["jail_flags"], value["chroot"]))
+                
             if "jail_flags" in value:
                 jail_flags = sorted(value["jail_flags"])
                 first = jail_flags[0]
